@@ -1,5 +1,6 @@
 /*
  * Copyright (C) 2003 Sun Microsystems, Inc.
+ * Copyright (C) 2004 Red Hat Inc.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -61,6 +62,9 @@ typedef struct
   GtkWidget      *status;
   GtkWidget      *received;
   GtkWidget      *sent;
+  GtkWidget      *signal_strength_frame;
+  GtkWidget      *signal_strength_bar;
+  GtkWidget      *signal_strength_label;
   GtkWidget      *configure_button;
 
   GtkWidget      *inet4_frame;
@@ -122,34 +126,36 @@ static inline void
 print_packets_string (GString *str,
 		      gulong   packets)
 {
-  g_string_printf (str, _("%lu packets"), packets);
+  g_string_printf (str, ngettext ("%lu packet", "%lu packets", packets), packets);
 }
 
 static inline void
 print_bytes_string (GString *str,
-		    guint64  bytes)
+		    gulong   bytes)
 {
-  if (bytes > 1 << 30)
-    {
-      bytes = (bytes * 10) / (1 << 30);
+  bytes *= 10;
 
-      g_string_append_printf (str, " (%lld.%lld Gb)", bytes / 10, bytes % 10);
+  if (bytes > 2 << 30)
+    {
+      bytes = bytes / (2 << 30);
+
+      g_string_append_printf (str, " (%ld.%ld Gb)", bytes / 10, bytes % 10);
     }
-  else if (bytes > 1 << 20)
+  else if (bytes > 2 << 20)
     {
-      bytes = (bytes * 10) / (1 << 20);
+      bytes = bytes / (2 << 20);
 
-      g_string_append_printf (str, " (%lld.%lld Mb)", bytes / 10, bytes % 10);
+      g_string_append_printf (str, " (%ld.%ld Mb)", bytes / 10, bytes % 10);
     }
-  else if (bytes > 1 << 10)
+  else if (bytes > 2 << 10)
     {
-      bytes = (bytes * 10) / (1 << 10);
+      bytes = bytes / (2 << 10);
 
-      g_string_append_printf (str, " (%lld.%lld Kb)", bytes / 10, bytes % 10);
+      g_string_append_printf (str, " (%ld.%ld Kb)", bytes / 10, bytes % 10);
     }
   else if (bytes >= 0)
     {
-      g_string_append_printf (str, " (%lld.%lld b)", bytes / 10, bytes % 10);
+      g_string_append_printf (str, " (%ld.%ld b)", bytes / 10, bytes % 10);
     }
 }
 
@@ -178,6 +184,31 @@ netstatus_dialog_update_activity (NetstatusDialogData *data)
   g_string_free (str, TRUE);
 }
 
+static void
+netstatus_dialog_update_signal_strength (NetstatusDialogData *data)
+{
+  if (netstatus_iface_get_is_wireless (data->iface))
+    {
+      int   signal_strength;
+      char *str;
+
+      gtk_widget_show (data->signal_strength_frame);
+
+      signal_strength = netstatus_iface_get_signal_strength (data->iface);
+
+      gtk_progress_bar_set_fraction (GTK_PROGRESS_BAR (data->signal_strength_bar),
+				     signal_strength == 0 ? 0.0 : signal_strength / 100.0);
+
+      str = g_strdup_printf ("%d%%\n", signal_strength);
+      gtk_label_set_text (GTK_LABEL (data->signal_strength_label), str);
+      g_free (str);
+    }
+  else
+    {
+      gtk_widget_hide (data->signal_strength_frame);
+    }
+}
+
 static void
 netstatus_dialog_update_inet4_support (NetstatusDialogData *data)
 {
@@ -300,12 +331,21 @@ netstatus_dialog_iface_state_changed (NetstatusIface      *iface,
   netstatus_dialog_update_device_support (data);
 }
 
+static void
+netstatus_dialog_iface_signal_strength_changed (NetstatusIface      *iface,
+						GParamSpec          *pspec,
+						NetstatusDialogData *data)
+{
+  netstatus_dialog_update_signal_strength (data);
+}
+
 static void
 netstatus_dialog_iface_name_changed (NetstatusIface      *iface,
 				     GParamSpec          *pspec,
 				     NetstatusDialogData *data)
 {
   netstatus_dialog_update_name (data);
+  netstatus_dialog_update_signal_strength (data);
 }
 
 static void
@@ -337,7 +377,7 @@ netstatus_dialog_response (GtkWidget *dialog,
 
 	screen = gtk_window_get_screen (GTK_WINDOW (dialog));
 
-	gnome_help_display_on_screen ("gnome-netstatus", NULL, screen, &error);
+	gnome_help_display_on_screen ("gnome-netstatus", "gnome-netstatus-props", screen, &error);
 
 	if (error)
 	  {
@@ -571,6 +611,7 @@ netstatus_dialog_setup_connection (NetstatusDialogData *data,
 
   icon = netstatus_icon_new (data->iface);
   netstatus_icon_set_tooltips_enabled (NETSTATUS_ICON (icon), FALSE);
+  netstatus_icon_set_show_signal (NETSTATUS_ICON (icon), FALSE);
   gtk_box_pack_end (GTK_BOX (hbox), icon, FALSE, TRUE, 4);
   gtk_widget_show (icon);
 
@@ -593,6 +634,17 @@ netstatus_dialog_setup_activity (NetstatusDialogData *data,
   netstatus_dialog_update_activity (data);
 }
 
+static void
+netstatus_dialog_setup_signal_strength_details (NetstatusDialogData *data,
+						GladeXML            *xml)
+{
+  data->signal_strength_frame = glade_xml_get_widget (xml, "signal_strength_frame");
+  data->signal_strength_bar   = glade_xml_get_widget (xml, "signal_strength_bar");
+  data->signal_strength_label = glade_xml_get_widget (xml, "signal_strength_label");
+
+  netstatus_dialog_update_signal_strength (data);
+}
+
 static void
 netstatus_dialog_setup_inet4_support (NetstatusDialogData *data,
 				      GladeXML            *xml)
@@ -698,6 +750,18 @@ netstatus_dialog_new (NetstatusIface *iface)
 					G_CALLBACK (netstatus_dialog_iface_name_changed),
 					data,
 					data->dialog);
+  
+  netstatus_connect_signal_while_alive (data->iface,
+					"notify::wireless",
+					G_CALLBACK (netstatus_dialog_iface_signal_strength_changed),
+					data,
+					data->dialog);
+
+  netstatus_connect_signal_while_alive (data->iface,
+					"notify::signal-strength",
+					G_CALLBACK (netstatus_dialog_iface_signal_strength_changed),
+					data,
+					data->dialog);
 
   g_signal_connect (data->dialog, "response",
 		    G_CALLBACK (netstatus_dialog_response), NULL);
@@ -707,6 +771,7 @@ netstatus_dialog_new (NetstatusIface *iface)
 
   netstatus_dialog_setup_connection (data, xml);
   netstatus_dialog_setup_activity (data, xml);
+  netstatus_dialog_setup_signal_strength_details (data, xml);
   netstatus_dialog_setup_inet4_support (data, xml);
   netstatus_dialog_setup_device_support (data, xml);
   netstatus_dialog_setup_configure_button (data, xml);
