@@ -1,5 +1,6 @@
 /*
  * Copyright (C) 2003 Sun Microsystems, Inc.
+ * Copyright (C) 2004 Red Hat Inc.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -28,12 +29,13 @@
 
 #include <stdio.h>
 #include <string.h>
+#include <math.h>
 #include <unistd.h>
 #include <errno.h>
 #include <glib.h>
 #include <libgnome/gnome-i18n.h>
 
-static inline int
+static inline gboolean
 parse_stats (char  *buf,
 	     int    prx_idx,
 	     int    ptx_idx,
@@ -95,11 +97,11 @@ parse_iface_name (const char *buf)
 }
 
 static inline void
-parse_header (char *buf,
-	      int  *prx_idx,
-	      int  *ptx_idx,
-	      int  *brx_idx,
-	      int  *btx_idx)
+parse_stats_header (char *buf,
+		    int  *prx_idx,
+		    int  *ptx_idx,
+		    int  *brx_idx,
+		    int  *btx_idx)
 {
   char *p;
   int   i;
@@ -151,6 +153,12 @@ netstatus_sysdeps_read_iface_statistics (const char  *iface,
   int   prx_idx, ptx_idx;
   int   brx_idx, btx_idx;
   char *error_message = NULL;
+
+  g_return_val_if_fail (iface != NULL, NULL);
+  g_return_val_if_fail (in_packets != NULL, NULL);
+  g_return_val_if_fail (out_packets != NULL, NULL);
+  g_return_val_if_fail (in_bytes != NULL, NULL);
+  g_return_val_if_fail (out_bytes != NULL, NULL);
   
   *in_packets  = -1;
   *out_packets = -1;
@@ -165,7 +173,7 @@ netstatus_sysdeps_read_iface_statistics (const char  *iface,
   fgets (buf, sizeof (buf), fh);
   fgets (buf, sizeof (buf), fh);
 
-  parse_header (buf, &prx_idx, &ptx_idx, &brx_idx, &btx_idx);
+  parse_stats_header (buf, &prx_idx, &ptx_idx, &brx_idx, &btx_idx);
   if (prx_idx == -1 || ptx_idx == -1 ||
       brx_idx == -1 || btx_idx == -1)
     return g_strdup (_("Could not parse /proc/net/dev. Unknown format."));
@@ -201,12 +209,141 @@ netstatus_sysdeps_read_iface_statistics (const char  *iface,
 					   buf, prx_idx, ptx_idx, brx_idx, btx_idx);
 	  continue;
 	}
+
+      break;
     }
 
   if ((*in_packets == -1 || *out_packets == -1 || *in_bytes == -1 || *out_bytes == -1) && !error_message)
     error_message = g_strdup_printf ("Could not find information on interface '%s' in /proc/net/dev", iface);
 
   rewind (fh);
+  fflush (fh);
+
+  return error_message;
+}
+
+static inline gboolean
+parse_wireless (char  *buf,
+		int    link_idx,
+		int   *link)
+{
+  char *p;
+  int   i;
+
+  p = strtok (buf, " \t\n");
+  for (i = 0; p; i++, p = strtok (NULL, " \t\n"))
+    {
+      if (i == link_idx)
+	*link = g_ascii_strtoull (p, NULL, 10);
+    }
+
+  if (i <= link_idx)
+    return FALSE;
+
+  return TRUE;
+}
+
+static inline int
+parse_wireless_header (char *buf)
+{
+  char *p;
+  int   i;
+
+  p = strtok (buf, "| \t\n");
+  p = strtok (NULL, "| \t\n"); /* Skip the first one */
+  for (i = 0; p; i++, p = strtok (NULL, "| \t\n"))
+    {
+      if (!strcmp (p, "link"))
+	{
+	  return i;
+	}
+    }
+
+  return -1;
+}
+
+static inline FILE *
+get_proc_net_wireless_fh (void)
+{
+  static FILE *retval = NULL;
+
+  if (retval != NULL)
+    return retval;
+
+  return retval = fopen ("/proc/net/wireless", "r");
+}
+
+char *
+netstatus_sysdeps_read_iface_wireless_details (const char *iface,
+					       gboolean   *is_wireless,
+					       int        *signal_strength)
+{
+  FILE *fh;
+  char  buf [512];
+  int   link_idx;
+  char *error_message = NULL;
+
+  g_return_val_if_fail (iface != NULL, NULL);
+  g_return_val_if_fail (is_wireless != NULL, NULL);
+  g_return_val_if_fail (signal_strength != NULL, NULL);
+
+  if (is_wireless)
+    *is_wireless = FALSE;
+  if (signal_strength)
+    *signal_strength = 0;
+  
+  fh = get_proc_net_wireless_fh ();
+  if (!fh)
+    return g_strdup_printf (_("Cannot open /proc/net/wireless: %s"),
+			    g_strerror (errno));
+
+  fgets (buf, sizeof (buf), fh);
+  fgets (buf, sizeof (buf), fh);
+
+  link_idx = parse_wireless_header (buf);
+  if (link_idx == -1)
+    return g_strdup (_("Could not parse /proc/net/wireless. Unknown format."));
+
+  while (fgets (buf, sizeof (buf), fh))
+    {
+      char *details;
+      char *name;
+      int   link;
+
+      name = buf;
+      while (g_ascii_isspace (name [0]))
+	name++;
+
+      details = parse_iface_name (name);
+      if (!details)
+	{
+	  if (!error_message)
+	    error_message = g_strdup_printf (_("Could not parse interface name from '%s'"), buf);
+	  continue;
+	}
+
+      if (strcmp (name, iface) != 0)
+	continue;
+
+      if (!parse_wireless (details, link_idx, &link))
+	{
+	  if (error_message)
+	    g_free (error_message);
+	  error_message = g_strdup_printf (_("Could not parse wireless details from '%s'. link_idx = %d;"),
+					   buf, link_idx);
+	  continue;
+	}
+
+      /* Stolen from the wireless applet */
+      *signal_strength = (int) rint ((log (link) / log (92)) * 100.0);
+      *signal_strength = CLAMP (*signal_strength, 0, 100);
+      *is_wireless = TRUE;
+
+      break;
+    }
+
+  rewind (fh);
+  fflush (fh);
 
   return error_message;
 }
@@ -261,6 +398,12 @@ netstatus_sysdeps_read_iface_statistics (const char *iface,
   char    *error_message = NULL;
   int      pipe_out;
 
+  g_return_val_if_fail (iface != NULL, NULL);
+  g_return_val_if_fail (in_packets != NULL, NULL);
+  g_return_val_if_fail (out_packets != NULL, NULL);
+  g_return_val_if_fail (in_bytes != NULL, NULL);
+  g_return_val_if_fail (out_bytes != NULL, NULL);
+
   *in_packets  = -1;
   *out_packets = -1;
   *in_bytes    = -1;
@@ -345,4 +488,21 @@ netstatus_sysdeps_read_iface_statistics (const char *iface,
   return error_message;
 }
 
+char *
+netstatus_sysdeps_read_iface_wireless_details (const char *iface,
+					       gboolean   *is_wireless,
+					       int        *signal_strength)
+{
+  g_return_val_if_fail (iface != NULL, NULL);
+  g_return_val_if_fail (is_wireless != NULL, NULL);
+  g_return_val_if_fail (signal_strength_wireless != NULL, NULL);
+
+  if (is_wireless)
+    *is_wireless = FALSE;
+  if (signal_strength)
+    *signal_strength = 0;
+
+  return NULL;
+}
+
 #endif /* !defined(__FreeBSD__) */
